#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include "udm_utils.h"
#include "udm_os.h"

int tz_offset=0;

char * UdmTrim(char *p, char *delim){
int len;
	len = strlen(p);
	while ((len > 0) && strchr(delim, p[len - 1] )) {
		p[len - 1] = '\0';
		len--;
	}
	while((*p)&&(strchr(delim,*p)))p++;
	return(p);
}

char * UdmRTrim(char* p, char *delim){
int len;
	len = strlen(p);
	while ((len > 0) && strchr(delim, p[len - 1] )) {
		p[len - 1] = '\0';
		len--;
	}
	return(p);
}


/* strtok_r clone */
char * UdmGetToken(char *s, const char *delim, char **last)
{
    char *spanp;
    int c, sc;
    char *tok;

    if (s == NULL && (s = *last) == NULL)
	return NULL;

cont:
    c = *s++;
    for (spanp = (char *)delim; (sc = *spanp++) != 0; )
    {
	if (c == sc)
	{
	    goto cont;
	}
    }

    if (c == 0)		/* no non-delimiter characters */
    {
	*last = NULL;
	return NULL;
    }
    tok = s - 1;

    for (;;)
    {
	c = *s++;
	spanp = (char *)delim;
	do
	{
	    if ((sc = *spanp++) == c)
	    {
		if (c == 0)
		{
		    s = NULL;
		}
		else
		{
		    char *w = s - 1;
		    *w = '\0';
		}
		*last = s;
		return tok;
	    }
	}
	while (sc != 0);
    }
}



char * UdmHtmlSpecialChars(char *str){
char *pos, *p;
int len;
	if(!str)return(NULL);
	len = strlen(str)+1;
	for(pos = str; *pos; pos++){
		switch (*pos){
			case '&': len+=4; break;
			case '"': len+=5; break;
			case '<':
			case '>': len+=3; break;
		}
	}
	p = (char*) malloc(len+1);*p = 0;
	for(pos=p; *str; str++) {
	switch (*str){
		case '&': strcat(p, "&amp;"); pos+=5; break;
		case '"': strcat(p, "&quot;");pos+=6; break;
		case '<': strcat(p, "&lt;");pos+=4; break;
		case '>': strcat(p, "&gt;");pos+=4; break;
		default:
			*pos = *str;
			pos++;*pos=0;
		}
	}
	return p;
}

char * UdmUnescapeCGIQuery(char *d,char *s){
int hi,lo=0;
char *dd;
	if((d==NULL)||(s==NULL))return(0);
	dd=d;
	while(*s){
		if(*s=='%'){
			if(strchr("0123456789",*(++s))) hi=*s-'0';
			else hi=*s-'A'+10;
			if(strchr("0123456789",*(++s))) lo=*s-'0';
			else lo=*s-'A'+10;
			*d=hi*16+lo;
		}else
		if(*s=='+'){
			*d=' ';
		}else{
			*d=*s;
		}
		s++; d++;
	}
	*d=0;return(dd);
}

char * UdmEscapeURL(char *d,char *s){
char *dd;
	if((d==NULL)||(s==NULL))return(0);
	dd=d;
	while(*s){
		if(strchr("%&<>+[](){}/?#'\"\\;,",*s)){
			sprintf(d,"%%%x",(int)*s);
			*(d+1)=toupper(*(d+1));
			*(d+2)=toupper(*(d+2));
			d+=2;
		}else
		if(*s==' '){
			*d='+';
		}else{
			*d=*s;
		}
		s++;d++;
	}
	*d=0;
	return(dd);
}


/* Oh,no! We have to remove parent level like /parent/../index.html from path
   Let's do it recursively! */
char * UdmRemove2Dot(char *path){
        char *ptr;
	char *tail;
    
        if(!(ptr=strstr(path,"../"))) return path;
	if(ptr==path) return path; /* How could it be? */
        tail=ptr+2;
	ptr--;
        *ptr=0;
	if(!(ptr=strrchr(path,'/'))) *path=0; else *ptr=0;
        path=strcat(path,tail);
	return UdmRemove2Dot(path);
}

/* This should convert Last-Modified time returned by webserver
 * to time_t (seconds since the Epoch). -kir
 */
time_t UdmHttpDate2Time_t(char * time_str){
/*
  HTTP applications have historically allowed three different formats
  for the representation of date/time stamps:
 
  Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
  Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
  Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
			
  The first format is preferred as an Internet standard and represents
  a fixed-length subset of that defined by RFC 1123 [8] (an update to
  RFC 822 [9]). The second format is in common use, but is based on the
  obsolete RFC 850 [12] date format and lacks a four-digit year.
  HTTP/1.1 clients and servers that parse the date value MUST accept
  all three formats (for compatibility with HTTP/1.0), though they MUST
  only generate the RFC 1123 format for representing HTTP-date values
  in header fields. See section 19.3 for further information.
*/						
    
	struct tm tim;
    
        UDMMEMZERO(&tim, sizeof(tim));
	if (!strptime(time_str, "%a, %d %b %Y %T %Z", &tim))
		if (!strptime(time_str, "%a, %d-%b-%y %T %Z", &tim))
			if (!strptime(time_str, "%a %b %d %T %Y", &tim))
				return (time_t)-1; /* error */
	
	return mktime(&tim)-tz_offset;
}

time_t UdmFTPDate2Time_t(char *date){
        struct tm tm_s;

        if (strlen(date)!=20)
                return -1;
        UDMMEMZERO(&tm_s, sizeof(tm_s));
        strptime(date+6, "%y%m%d%H%M%S", &tm_s);
        return mktime(&tm_s)-tz_offset;
}

void UdmTime_t2HttpStr(time_t t, char *str){
    struct tm *time;
    if (t==0)
	*str='\0';
    else{
	/* FIXME: I suspect that gmtime IS NOT REENTRABLE!
	 */
	time=gmtime(&t);
	if (!strftime(str, UDM_MAXTIMESTRLEN, "%a, %d %b %Y  %H:%M:%S %Z", time))
	    *str='\0';
	/* end of BAD code */
    }
}

/* Find out tz_offset for the functions above
 */
int UdmInitTZ(void){
        time_t tt;
	struct tm *tms;

	tms=localtime(&tt);
	/* localtime sets the external variable timezone */
	tz_offset=timezone;
	return 0;
#if 0

#ifdef HAVE_TM_GMTOFF
	time_t clock;
	struct tm *tim;
        
	clock=time(NULL);
	tim=localtime(&clock);
        tz_offset=tim->tm_gmtoff;
	return 0;
#else                                                                           
	struct tm *t;
	int days, hours, minutes;

	tt= time(NULL);
        gmt = *gmtime(&tt);
        t = localtime(&tt);
        days = t->tm_yday - gmt.tm_yday;
        hours = ((days < -1 ? 24 : 1 < days ? -24 : days * 24)
                + t->tm_hour - gmt.tm_hour);
        minutes=hours * 60 + t->tm_min - gmt.tm_min;
        tz_offset=minutes*60;
	return 0;
#endif /* HAVE_TM_GMTOFF */

#endif /* if 0 */

/* ONE MORE VARIANT - how many do we have?
    struct timezone tz;

    gettimeofday(NULL, &tz);
    tz_offset=tz.tz_minuteswest*60;
    return 0;
*/
}
